#!/usr/bin/env python
# _*_ coding: utf-8 _*_
# @Time : 2022/6/5 16:01
# @Author : Rongrui Zhan
# @desc : 本代码未经授权禁止商用
__version__ = "0.1.0"
from .nets import EEGInception, EEGSpatialLSTM
from .dataset import ERPDataset
